#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// What are the structures

#ifndef SIZE
#define SIZE 10
#endif

typedef struct string {
    char *str; // it will hold '\0' anyway
    int len;
} string;
void free_string(string *str);

void freeString(string *str);

int distance(const char *str, const char *str2);

char *getLine(FILE *fd);
string *getLine2(FILE *fd);
int main(int argc, char *argv[])
{
    int ret = 0;
    char *fname = argc > 1 ? argv[1] : NULL;

    if (!fname) {
        fprintf(stderr, "Provide filename!\n");
        return 101;
    }
    FILE *fd = fopen(fname, "r");

    if (!fd) {
        fprintf(stderr, "ERROR: Cannot open file \"%s\"!\n", fname);
        return 102;
    }

    string *str1 = getLine2(fd);
    string *str2 = getLine2(fd);

    if (str1 == NULL || str2 == NULL) {
        fprintf(stderr, "ERROR: Cannot read line\n");
    } else {
        printf("Line1: \"%s\" len: %d strlen: %lu\n", str1->str, str1->len, strlen(str1->str));
        printf("Line2: \"%s\" len: %d strlen: %lu\n", str2->str, str2->len, strlen(str2->str));
    }

    if (str1) {
        freeString(str1);
    }
    if (str2) {
        freeString(str2);
    }

    fclose(fd);
    return ret;
}

void freeString(string *str)
{
    if (str) {
        if (str->str) {
            free(str->str);
        }
        free(str);
    }
}

void *myMalloc(size_t size)
{
    void *ret = malloc(size);
    if (!ret) {
        fprintf(stderr, "ERROR: cannot malloc\n");
        exit(-1);
    }
    return ret;
}

string *getLine2(FILE *fd)
{
    string *ret = myMalloc(sizeof(string));

    ret->str = getLine(fd);
    if (ret->str) {
        ret->len = strlen(ret->str);
    } else {
        free(ret);
        ret = NULL;
    }
    return ret;
}

char *getLine(FILE *fd)
{
    char *str = myMalloc(SIZE + 1);
    int capacity = SIZE;
    int len = 0;
    int c;

    while ((c = getc(fd)) != EOF && c != '\n') {
        if (len == capacity) { // enlarge str
            char *t = realloc(str, capacity + SIZE + 1);
            if (t == NULL) {
                free(str);
                fprintf(stderr, "ERROR: Cannot reallocate!\n");
                exit(-1);
            }
            str = t;
        }
        str[len++] = c;
    } //end while
    if (len > 0) {
        str[len] = '\0'; // during the live session on 2020-03-17 there was a bug in str[len+1] = '\0', it is not necessary to +1.
    } else {
        free(str);
        str = NULL;
    }
    return str;
}
